/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser.failure.message.generator;

import com.liferay.jenkins.results.parser.JenkinsResultsParserUtil;

import org.apache.tools.ant.Project;

/**
 * @author Kevin Yen
 * @author Peter Yoo
 */
public class FailureMessageUtil {

	public static String getFailureMessage(Project project, String buildURL)
		throws Exception {

		String consoleOutput = JenkinsResultsParserUtil.toString(
			JenkinsResultsParserUtil.getLocalURL(
				buildURL + "/logText/progressiveText"));

		for (FailureMessageGenerator failureMessageGenerator :
				_failureMessageGenerators) {

			String message = failureMessageGenerator.getMessage(
				buildURL, consoleOutput, project.getProperties());

			if (message != null) {
				return message;
			}
		}

		return _genericFailureMessageGenerator.getMessage(
			buildURL, consoleOutput, project.getProperties());
	}

	private static final FailureMessageGenerator[] _failureMessageGenerators = {
		new PoshiValidationFailureMessageGenerator(),

		new DownstreamFailureMessageGenerator(),

		new IntegrationTestTimeoutFailureMessageGenerator(),
		new LocalGitMirrorFailureMessageGenerator(),
		new PluginFailureMessageGenerator(),
		new PluginGitIDFailureMessageGenerator(),
		new RebaseFailureMessageGenerator(),
		new SemanticVersioningFailureMessageGenerator(),
		new SourceFormatFailureMessageGenerator()
	};

	private static final GenericFailureMessageGenerator
		_genericFailureMessageGenerator = new GenericFailureMessageGenerator();

}